Package: DiffBind
Type: Package
Version: 3.23.3
Title: Differential Binding Analysis of ChIP-Seq Peak Data
Authors@R: c(person("Rory","Stark", email="bioconductor@starkhome.com",
                  role = c("aut", "cre")),
             person("Gord","Brown",email="DECEASED",
                  role="aut"))
Description: Compute differentially bound sites from multiple ChIP-seq
        experiments using affinity (quantitative) data. Also enables
        occupancy (overlap) analysis and plotting functions.
License: Artistic-2.0
LazyLoad: yes
Depends: R (>= 4.0), GenomicRanges, SummarizedExperiment
Imports: RColorBrewer, amap, gplots, grDevices, limma,
        GenomicAlignments, locfit, stats, utils, IRanges, lattice,
        systemPipeR, tools, Rcpp, dplyr, ggplot2, BiocParallel,
        parallel, S4Vectors, Rsamtools (>= 2.13.1), DESeq2, methods,
        graphics, ggrepel, apeglm, ashr, GreyListChIP
Suggests: BiocStyle, testthat, xtable, rgl, XLConnect, edgeR, csaw,
        BSgenome, GenomeInfoDb, rtracklayer, grid
LinkingTo: Rhtslib (>= 1.99.1), Rcpp
SystemRequirements: GNU make
Collate: core.R parallel.R model.R counts.R contrast.R normalize.R
        analyze.R analyze_deseq2.R analyze_edgeR.R blacklist.R report.R
        plots.R plotProfile.R io.R helper.R utils.R RcppExports.R
        cpp_wrapper.R DBA.R
biocViews: Sequencing, ChIPSeq,ATACSeq, DNaseSeq, MethylSeq, RIPSeq,
        DifferentialPeakCalling, DifferentialMethylation,
        GeneRegulation, HistoneModification, PeakDetection,
        BiomedicalInformatics, CellBiology, MultipleComparison,
        Normalization, ReportWriting, Epigenetics, FunctionalGenomics
URL: https://bioconductor.org/packages/DiffBind/
git_url: https://git.bioconductor.org/packages/DiffBind
git_branch: devel
git_last_commit: a26ef8d
git_last_commit_date: 2026-07-01
Repository: Bioconductor 3.24
Date/Publication: 2026-07-05
NeedsCompilation: yes
Packaged: 2026-07-05 21:26:26 UTC; biocbuild
Author: Rory Stark [aut, cre],
  Gord Brown [aut]
Maintainer: Rory Stark <bioconductor@starkhome.com>
